package cn.com.ty.lift.business.maintenance.service;

import cn.com.ty.lift.business.maintenance.dao.entity.model.dto.MaintenancePlanMonthTaskNum;
import cn.com.ty.lift.business.maintenance.dao.entity.model.request.PropertyMaintenanceVO;
import cn.com.ty.lift.business.maintenance.dao.entity.model.response.PropertyCalendarResponse;
import cn.com.ty.lift.business.maintenance.dao.entity.model.response.PropertyMtResponse;
import cn.com.ty.lift.business.maintenance.dao.mapper.PropertyMaintenanceMapper;
import cn.com.ty.lift.common.constants.ApiConstants;
import cn.com.xwy.boot.web.dto.RestResponse;
import com.baomidou.mybatisplus.core.metadata.IPage;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

@Service
@AllArgsConstructor
@Slf4j
public class PropertyMaintenanceService {
    private final PropertyMaintenanceMapper propertyMaintenanceMapper;

    /**
     * @param propertyMaintenanceVO userId用户id，yearMonth年月 例202005即2020年5月
     * @return 日历
     */
    public RestResponse<?> queryMaintenancePlanMonth(PropertyMaintenanceVO propertyMaintenanceVO) {
        List<MaintenancePlanMonthTaskNum> planNums = propertyMaintenanceMapper.queryMaintenancePlanMonth(propertyMaintenanceVO.getUserId(), propertyMaintenanceVO.getYearMonth());
        if (planNums.isEmpty()) {
            return RestResponse.success();
        }
        log.info(planNums.toString());
        List<PropertyCalendarResponse> calendar = new ArrayList<>();
        //循环每日，每种状态的维保计划,计划超期状态不返回数据库的-1，返回3
        for (MaintenancePlanMonthTaskNum planNum : planNums) {
            if (planNum.getStatus().equals(-1)) {
                planNum.setStatus(3);
            }
            numToCalendar(calendar, planNum);
        }
        return RestResponse.success(calendar);
    }

    /**
     * 维保计划每日每个状态的数量转前端日历所需格式
     *
     * @param calendar 日历
     * @param planNum  计划数量
     */
    private void numToCalendar(List<PropertyCalendarResponse> calendar, MaintenancePlanMonthTaskNum planNum) {
        boolean hasDay = false;
        boolean hasStatus = false;
        //遍历每日
        for (PropertyCalendarResponse day : calendar) {
            if (day.getKey().equals(planNum.getDay())) {
                hasDay = true;
                //遍历每日的每个状态
                for (PropertyCalendarResponse.Values status : day.getValue()) {
                    if (status.getKey().equals(planNum.getStatus())) {
                        hasStatus = true;
                        status.setValue(planNum.getCount());
                    }
                }
                //如果遍历到的日期，没有这个状态，新增这个状态
                if (!hasStatus) {
                    PropertyCalendarResponse.Values status = new PropertyCalendarResponse.Values();
                    status.setKey(planNum.getStatus());
                    status.setValue(planNum.getCount());
                    day.getValue().add(status);
                }
            }
        }
        //如果日历中没有这个日期，新增这个日期
        if (!hasDay) {
            PropertyCalendarResponse day = new PropertyCalendarResponse();
            day.setKey(planNum.getDay());
            List<PropertyCalendarResponse.Values> statuses = day.getValue();
            PropertyCalendarResponse.Values status = new PropertyCalendarResponse.Values();
            status.setKey(planNum.getStatus());
            status.setValue(planNum.getCount());
            statuses.add(status);
            day.setValue(statuses);
            calendar.add(day);
        }
    }

    /**
     * @param propertyMtResponsePage 页码，页大小
     * @param propertyMaintenanceVO  请求参数
     * @return 分页查询某人加入的项目下电梯某天(待保养 / 保养中 / 已完成 / 超期 / 法规超期)的维保任务
     */
    public IPage<PropertyMtResponse> propertyMtResponsePage(IPage<PropertyMtResponse> propertyMtResponsePage, PropertyMaintenanceVO propertyMaintenanceVO) {
        IPage<PropertyMtResponse> page;
        Integer maintenanceStatus = ApiConstants.Maintenance.WAITING_MAINTENANCE;
        switch (propertyMaintenanceVO.getType()) {
            case 1:
                maintenanceStatus = ApiConstants.Maintenance.WAITING_MAINTENANCE;
                break;
            case 2:
                maintenanceStatus = ApiConstants.Maintenance.MAINTENANCE_DOING;
                break;
            case 3:
                maintenanceStatus = ApiConstants.Maintenance.COMPLETE;
                break;
            case 4:
                maintenanceStatus = ApiConstants.Maintenance.OVERDUE;
                break;
            case 5:
                maintenanceStatus = ApiConstants.Maintenance.LAWS_STIPULATION_OVERDUE;
                break;
        }
        if (maintenanceStatus.equals(ApiConstants.Maintenance.LAWS_STIPULATION_OVERDUE)) {
            page = propertyMaintenanceMapper
                    .queryMaintenancePlanEveryDayOverdue(
                            propertyMtResponsePage,
                            propertyMaintenanceVO.getPlanDate(),
                            propertyMaintenanceVO.getUserId());
        } else {
            page = propertyMaintenanceMapper
                    .queryMaintenancePlanEveryDay(
                            propertyMtResponsePage,
                            maintenanceStatus,
                            propertyMaintenanceVO.getPlanDate(),
                            propertyMaintenanceVO.getUserId());
        }
        return page;
    }
}