package cn.com.ty.lift.common.constants;

public interface SqlConstants {
    //根据用户id和维保计划时间和状态查询用户那一天的各种类型维保任务(待保养/保养中/已完成/计划超期)
    String QUERY_MAINTENANCE_PLAN_EVERYDAY =
            "select         p.project_name projectName," +
                    "       l.registration_code registrationCode," +
                    "       l.device_position devicePosition," +
                    "       l.lift_type liftType," +
                    "       l.category category," +
                    "       ui.name workerName," +
                    "       mp.plan_date planDate" +
                    "          from maintenance_plan mp" +
                    "           left join lift l on mp.lift_id = l.id" +
                    "           left join user_info ui on mp.worker_id = ui.user_id" +
                    "           left join project p on mp.project_id = p.id" +
                    "       where mp.plan_date = #{planDate}" +
                    "             and mp.status = #{status}" +
                    "             and mp.project_id in (select project_id from project_user where user_id = #{userId})";

    //查询法规超期维保列表。
    String QUERY_MAINTENANCE_PLAN_EVERYDAY_OVERDUE =
            "select         mp.status           planStatus," +
                    "       l.id                liftId," +
                    "       p.project_name      projectName," +
                    "       l.registration_code registrationCode," +
                    "       l.device_position   devicePosition," +
                    "       l.lift_type         liftType," +
                    "       l.category          category," +
                    "       ui.name             name" +
                    "   from maintenance_plan mp" +
                    "         left join lift l on mp.lift_id = l.id" +
                    "         left join user_info ui on mp.worker_id = ui.user_id" +
                    "         left join project p on mp.project_id = p.id" +
                    "   where plan_date = #{planDate}" +
                    "   and work_date is null" +
                    "   and TimeStampDiff(DAY, DATE_FORMAT(plan_date, '%Y%m%d'), DATE_FORMAT(CURDATE(), '%Y%m%d')) > 15" +
                    "   and mp.project_id in (select project_id from project_user where user_id = #{userId})";

    //扫码，查询电梯信息
    String QUERY_LIFT_INFO_BY_LIFT_ID =
            "<script>" +
                    "select l.id                id," +
                    "       l.registration_code registrationCode," +
                    "       l.device_position   devicePosition," +
                    "       p.project_name      projectName," +
                    "       if(l.lift_type = 1, '直梯', '扶梯')         liftType," +
                    "       ui.name             workerName," +
                    "       ui.mobile           workerMobile," +
                    "       p.project_status    projectStatus" +
                    "   from project_lift_relevance plr" +
                    "         left join lift l on plr.lift_id = l.id" +
                    "         left join project p on plr.project_id = p.id" +
                    "         left join user_info ui on plr.worker_id = ui.user_id" +
                    "   where l.id = #{liftId}" +
                    "       and p.project_status = '2'" +
                    "</script>";

    //扫码，查询维保记录列表
    String QUERY_MAINTENANCE_RECORD_BY_LIFT_ID =
            "select         mr.id                 recordId," +
                    "       mr.plan_date          planDate," +
                    "       mr.work_date          workDate," +
                    "       ui.name               workerName," +
                    "       mr.maintenance_advice maintenanceAdvice," +
                    "       e.advice              consumerAdvice," +
                    "       mr.status             status," +
                    "       mr.lift_id            liftId," +
                    "       mr.lift_type          liftType" +
                    "   from maintenance_record mr" +
                    "         left join user_info ui on mr.worker_id = ui.user_id" +
                    "         left join evaluation e on mr.id = e.record_id and e.source = '1'" +
                    "   where mr.lift_id = #{liftId}" +
                    "       order by mr.work_date desc";

    //物管，根据用户id和月份，查询那一月每天的(待保养/保养中/已完成/计划超期)的数量
    String QUERY_MAINTENANCE_PLAN_MONTH_TASK_NUM =
            "select     DATE_FORMAT(plan_date, '%d') day, status status, count(status) count" +
                    "   from maintenance_plan" +
                    "   where project_id in (select project_id from project_user where user_id = #{userId})" +
                    "       and DATE_FORMAT(plan_date, '%Y-%m') = #{yearMonth}" +
                    "       group by plan_date, status";
}
